<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:template match="course">
<html>

<!--
STUDENT GRADES SPREADSHEET

  ns:   number of students in class
  hwXm: homework maximum
  exXm: exam maximum
  hwXw: homework weight
  exXw: exam weight
  hwXa: homework average
  exXa: exam average
-->

<!--- count the students -->

<xsl:variable name="ns">
 <xsl:value-of select="count(student)"/>
</xsl:variable>

<!--- homework and exam max -->

<xsl:variable name="hw1m">
 <xsl:value-of select="hw1/@max"/>
</xsl:variable>
<xsl:variable name="hw2m">
 <xsl:value-of select="hw2/@max"/>
</xsl:variable>
<xsl:variable name="ex1m">
 <xsl:value-of select="ex1/@max"/>
</xsl:variable>
<xsl:variable name="ex2m">
 <xsl:value-of select="ex2/@max"/>
</xsl:variable>

<!--- homework and exam weights -->

<xsl:variable name="hw1w">
 <xsl:value-of select="hw1/@weight"/>
</xsl:variable>
<xsl:variable name="hw2w">
 <xsl:value-of select="hw2/@weight"/>
</xsl:variable>
<xsl:variable name="ex1w">
 <xsl:value-of select="ex1/@weight"/>
</xsl:variable>
<xsl:variable name="ex2w">
 <xsl:value-of select="ex2/@weight"/>
</xsl:variable>

<!--- homework and exam normalization factor-->

<xsl:variable name="norm">
 <xsl:value-of select="$hw1m*$hw1w + $hw2m*$hw2w 
                      +$ex1m*$ex1w + $ex2m*$ex2w"
  />
</xsl:variable>

<!--- homework and exam averages-->

<xsl:variable name="hw1a">
 <xsl:value-of select="sum(student/hw1/@grade) div $ns"/>
</xsl:variable>
<xsl:variable name="hw2a">
 <xsl:value-of select="sum(student/hw2/@grade) div $ns"/>
</xsl:variable>
<xsl:variable name="ex1a">
 <xsl:value-of select="sum(student/ex1/@grade) div $ns"/>
</xsl:variable>
<xsl:variable name="ex2a">
 <xsl:value-of select="sum(student/ex2/@grade) div $ns"/>
</xsl:variable>

<!-- generate html code -->

<head>
 <title>
 <xsl:value-of select="@name"/> Grades
 </title>
</head>

<!-- html styles -->

<style type="text/css">

table.grades {
        border-width: 1px;
        border-spacing: 2px;
        border-style: outset;
        border-color: gray;
        border-collapse: separate;
        background-color: white;
        font-size:12px;
}
table.grades th {
        border-width: 1px;
        padding: 1px;
        border-style: inset;
        border-color: gray;
        background-color: white;
        color: maroon;
        font-size:14px;
}
table.grades td {
        border-width: 1px;
        padding: 1px;
        border-style: inset;
        border-color: gray;
}
</style>

<body>

<!-- course name and term -->

<xsl:value-of select="@name"/>,
<xsl:value-of select="@term"/>
<br/><br/>

<!-- table -->

<table class="grades">

<tr>
<th>Student</th>
<th>Hw 1</th>
<th>Hw 2</th>
<th>Ex 1</th>
<th>Ex 2</th>
<th>Score</th>
<th>Grade</th>
</tr>

<xsl:for-each select ="student">
<xsl:sort select="firstname" order="ascending"/>
<tr>

  <td align="left">
    <xsl:value-of select="lastname"/>  &#173;
    <xsl:value-of select="firstname"/> &#173;
    <xsl:value-of select="middlename"/>
  </td>
  <td align="right"><xsl:value-of select="hw1/@grade"/></td>
  <td align="right"><xsl:value-of select="hw2/@grade"/></td>
  <td align="right"><xsl:value-of select="ex1/@grade"/></td>
  <td align="right"><xsl:value-of select="ex2/@grade"/></td>
  <td align="right">
  <xsl:variable name="grdn">
    <xsl:value-of select=" (( hw1/@grade * $hw1w + hw2/@grade * $hw2w
                           + ex1/@grade * $ex1w + ex2/@grade * $ex2w 
                            ) * 100 ) div $norm"/>
    </xsl:variable>
    <xsl:value-of select='format-number($grdn,"##.##")'/>
   </td>
  <td align="center">
   <xsl:variable name="grdn">
    <xsl:value-of select=" (( hw1/@grade * $hw1w + hw2/@grade * $hw2w
                          + ex1/@grade * $ex1w + ex2/@grade * $ex2w 
                           ) * 100 ) div $norm"/>
    </xsl:variable>
   <xsl:choose>
     <xsl:when test="($grdn &gt; 95) and ($grdn &lt; 101)">A</xsl:when>
     <xsl:when test="($grdn &gt; 90) and ($grdn &lt;  95)">B</xsl:when>
     <xsl:when test="($grdn &gt; 85) and ($grdn &lt;  90)">C</xsl:when>
     <xsl:when test="($grdn &gt; 80) and ($grdn &lt;  85)">D</xsl:when>
     <xsl:otherwise>F</xsl:otherwise>
  </xsl:choose>
  </td>

</tr>
</xsl:for-each>


<!-- maximum -->

<tr bgcolor="ghostwhite">
<td align="center">Maximum</td>
<td align="right"><xsl:value-of select="$hw1m"/></td>
<td align="right"><xsl:value-of select="$hw2m"/></td>
<td align="right"><xsl:value-of select="$ex1m"/></td>
<td align="right"><xsl:value-of select="$ex2m"/></td>
<td align="right"></td>
<td align="right"></td>
</tr>

<!-- averages -->

<tr bgcolor="ghostwhite">
<td align="center">Average </td>
<td align="right"><xsl:value-of select='format-number($hw1a,"##.##")'/></td>
<td align="right"><xsl:value-of select='format-number($hw2a,"##.##")'/></td>
<td align="right"><xsl:value-of select='format-number($ex1a,"##.##")'/></td>
<td align="right"><xsl:value-of select='format-number($ex2a,"##.##")'/></td>
<td align="right"></td>
<td align="right"></td>
</tr>

<!-- end of a table -->

</table>

<br/>
<xsl:value-of select="count(student)"/> students in this class
<!--
<xsl:value-of select="$ns"/> students in this class
-->
<br/><br/>

</body>

</html>
</xsl:template>

<!--Ignore the text nodes (the white spaces)
<xsl:template match="text()">
</xsl:template>
-->

</xsl:stylesheet>
