      subroutine elm_line1
     +
     +  (N
     +  ,ratio
     +  ,X1,Y1
     +  ,X2,Y2
     +  ,sinit
     +  ,Xe,Ye,se
     +  ,Xm,Ym,sm
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-----------------------------------------------------
c Unsymmetrical discretization of a line segment
c into a graded mesh of N elements
c
c  X1,Y1: coordinates of the first point
c  X2,Y2: coordinates of the last point
c
c  ratio: 
c
c  alpha: geometric factor ratio
c
c  sinit: specified arc length at (X1, Y1)
c
c  se: arc length at the element end-nodes
c  sm: arc length at the element mid-nodes
c
c  xe,ye: end nodes
c  xm,ym: mid nodes
c
c-----------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension Xe(129),Ye(129),se(129)
      Dimension Xm(128),Ym(128),sm(128)

c------------
c one element
c------------

      If(N.eq.1) then
       xe(1) = X1 
       ye(1) = Y1 
       xe(2) = X2 
       ye(2) = Y2 
       se(1) = sinit
       se(2) = se(1)+Dsqrt( (X2-X1)**2+(Y2-Y1)**2)
       Go to 99
      End If

c--------------
c many elements
c--------------

      If(ratio.eq.1.000) then
        alpha  = 1.0D0
        factor = 1.0D0/N
      Else
        texp   = 1.0D0/(N-1.0D0)
        alpha  = ratio**texp
        factor = (1.0D0-alpha)/(1.0D0-alpha**N)
      End If

      write (6,*) ratio
      write (6,*) "elm_line: Geometric ratio: ",alpha

      deltax = (x2-x1) * factor   ! x length of first element
      deltay = (y2-y1) * factor   ! y length of first element 

      Xe(1) = X1     ! first point
      Ye(1) = Y1
      se(1) = sinit

      Do i=2,N+1
        Xe(i)  = Xe(i-1)+deltax
        Ye(i)  = Ye(i-1)+deltaY
        se(i)  = se(i-1)+sqrt(deltax**2+deltay**2)
        deltax = deltax*alpha
        deltay = deltay*alpha
      End Do

c-----
c Done
c-----

  99  Continue

c-------------------
c compute mid-points
c-------------------

      Do i=1,N
       xm(i) = 0.5D0*(xe(i)+xe(i+1))
       ym(i) = 0.5D0*(ye(i)+ye(i+1))
       sm(i) = 0.5D0*(se(i)+se(i+1))
      End Do

c-----
c Done
c-----

      Return
      End
